@@ -1,4 +1,4 @@
-use crate::{zest_block::ZestBlock, zest_hash::zest4096};
+use crate::{zest_block::ZestBlock, zest_digest::zest_digest, zest_hash::zest4096};
 
 //Can be parallelized. Keep in mind that the key,
 //init_ctr_enc, init_result_dgst, and init_ctr_dgst
@@ -7,37 +7,39 @@ use crate::{zest_block::ZestBlock, zest_hash::zest4096};
 //The key should be unique for every single message (as a nonce in itself)
 //for maximum protection.
 
+#[derive(Debug)]
+pub enum DecryptErr {
+    NoDigest, InvalidDigest
+}
+
 pub fn zest_dec(
     cipher_blocks: Vec<ZestBlock>,
     key: ZestBlock,
     init_ctr_dec: Option<ZestBlock>,
     init_result_dgst: Option<ZestBlock>,
     init_ctr_dgst: Option<ZestBlock>,
-) -> Option<Vec<ZestBlock>> {
+) -> Result<Vec<ZestBlock>, DecryptErr> {
     //Variables needed for decryption
     let mut text_out = vec![];
     let mut counter = init_ctr_dec.unwrap_or_else(ZestBlock::new);
-    //Variables needed for calculating digest
-    let mut result_dgst = init_result_dgst.unwrap_or_else(ZestBlock::new);
-    let mut ctr_dgst = init_ctr_dgst.unwrap_or_else(ZestBlock::new);
     for block in cipher_blocks.iter().take(cipher_blocks.len() - 1) {
         //Generate CSPRNG data stream and extract data.
         let mut hashed_out = zest4096(key ^ counter);
         hashed_out ^= *block;
         counter.increment_block();
         text_out.push(hashed_out);
-
-        //Compute partial digest
-        result_dgst ^= zest4096(zest4096(hashed_out ^ ctr_dgst) ^ ctr_dgst);
-        ctr_dgst.increment_block();
     }
     //Check if there *is* a encoded digest attached to the recieved data
-    if let Some(digest_enc) = cipher_blocks.last() {
-        if *digest_enc == result_dgst {
-            return Some(text_out);
+    if let Some(enc_digest_enc) = cipher_blocks.last() {
+        //Decrypt digest block
+        let cipher_stream = zest4096(key ^ counter);
+        let digest_enc = *enc_digest_enc ^ cipher_stream;
+        if digest_enc == zest_digest(text_out.clone(), init_result_dgst, init_ctr_dgst) {
+            return Ok(text_out);
+        } else {
+            return Err(DecryptErr::InvalidDigest);
         }
+    } else {
+        return Err(DecryptErr::NoDigest);
     }
-    //If there is no digest, or the digest matched is not identical to the
-    //calculated digest, then the message can't be verified.
-    None
 }
