@@ -1,4 +1,4 @@
-use crate::{zest_block::ZestBlock, zest_hash::zest4096};
+use crate::{zest_block::ZestBlock, zest_digest::zest_digest, zest_hash::zest4096};
 
 //Can be parallelized. Keep in mind that the key,
 //init_ctr_enc, init_result_dgst, and init_ctr_dgst
@@ -17,22 +17,17 @@ pub fn zest_enc(
     //Variables needed for encryption
     let mut cipher_text = vec![];
     let mut counter = init_ctr_enc.unwrap_or_else(ZestBlock::new);
-    //Variables needed for decryption
-    let mut result_dgst = init_result_dgst.unwrap_or_else(ZestBlock::new);
-    let mut ctr_dgst = init_ctr_dgst.unwrap_or_else(ZestBlock::new);
-    for text_block in padded_text {
+    for text_block in padded_text.clone() {
         //Generate next CSPRNG data stream and squash in the cipher text.
         let mut cipher_block = zest4096(key ^ counter);
         cipher_block ^= text_block;
         counter.increment_block();
         cipher_text.push(cipher_block);
-
-        //Compute partial digest
-        result_dgst ^= zest4096(zest4096(text_block ^ ctr_dgst) ^ ctr_dgst);
-        ctr_dgst.increment_block();
     }
-    //Add the computed digest to the block stream
-    cipher_text.push(result_dgst);
+    //Add the computed digest to the block stream, and encrypt it as well.
+    let digest = zest_digest(padded_text, init_result_dgst, init_ctr_dgst);
+    let cipher_stream = zest4096(key ^ counter);
+    cipher_text.push(digest ^ cipher_stream);
     //Return the cipher text + digest
     cipher_text
 }
