@@ -1,27 +1,17 @@
-use crate::zest_block::ZestBlock;
+use crate::zest_block::{bytes_to_blocks, ZestBlock};
 
 // Takes in arbitrary length input and pads it to needed length.
 #[inline(always)]
-pub fn pad(input: &[u8]) -> Vec<ZestBlock> {
+pub fn pad_bytes(input: &[u8]) -> Vec<ZestBlock> {
     let mut un_padded = input.to_vec();
     un_padded.push(0xFF);
     let alignment_add = 0x200 - (un_padded.len() & 0x1FF);
     un_padded.resize(un_padded.len() + alignment_add, 0);
-    to_blocks(un_padded)
+    bytes_to_blocks(un_padded)
 }
 
 // Takes in blocks and de-pads it back to original size.
 #[inline(always)]
-pub fn de_pad(input: &mut Vec<u8>) {
+pub fn depad_bytes(input: &mut Vec<u8>) {
     while input.pop() == Some(0) {}
 }
-
-#[inline(always)]
-fn to_blocks(input: Vec<u8>) -> Vec<ZestBlock> {
-    let mut blocks: Vec<ZestBlock> = vec![];
-    for block in input.chunks_exact(512) {
-        let block_arr: [u8; 512] = block.try_into().unwrap();
-        blocks.push(block_arr.into());
-    }
-    blocks
-}
